import { gql } from 'apollo-server-express';

export const typeDefs = gql`
  type Group {
    id: ID!
    name: String!
    parent_id: ID
    tasks: [Task!]!
    created_at: String!
    updated_at: String!
  }

  type Task {
    id: ID!
    name: String!
    group_id: ID!
    group: Group!
    steps: [Step!]!
    notes: [Note!]!
    print_count: Int!
    last_printed_at: String
    printHistory: [PrintHistory!]!
    created_at: String!
    updated_at: String!
  }

  type Step {
    id: ID!
    name: String!
    instructions: String!
    task_id: ID!
    task: Task!
    order: Int!
    notes: [Note!]!
    print_count: Int!
    last_printed_at: String
    printHistory: [PrintHistory!]!
    images: [Image!]!
    created_at: String!
    updated_at: String!
  }

  type Image {
    id: ID!
    step_id: ID!
    step: Step!
    original_path: String!
    bw_path: String!
    order: Int!
    created_at: String!
    updated_at: String!
  }

  type User {
    id: ID!
    name: String!
    created_at: String!
    updated_at: String!
  }

  type Note {
    id: ID!
    content: String!
    step_id: ID!
    step: Step!
    user: User!
    createdBy: User!
    created_at: String!
    updated_at: String!
  }

  type PrintHistory {
    id: ID!
    user: User!
    task: Task
    step: Step
    printed_at: String!
    created_at: String!
    updated_at: String!
  }

  type Query {
    groups: [Group!]!
    group(id: ID!): Group
    tasks(groupId: ID!): [Task!]!
    task(id: ID!): Task
    steps(taskId: ID!): [Step!]!
    step(id: ID!): Step
    recentTasks: [Task!]!
    frequentTasks: [Task!]!
    users: [User!]!
    user(id: ID!): User
    notes(stepId: ID!): [Note!]!
    printHistory(taskId: ID, stepId: ID): [PrintHistory!]!
  }

  type Mutation {
    createGroup(name: String!, parentId: ID): Group!
    createTask(name: String!, groupId: ID!): Task!
    createStep(name: String!, instructions: String!, taskId: ID!, order: Int!): Step!
    createImage(stepId: ID!, originalPath: String!, bwPath: String!, order: Int!): Image!
    createUser(name: String!): User!
    createNote(content: String!, stepId: ID!): Note!
    printTask(id: ID!, userId: ID!): Task!
    printStep(id: ID!, userId: ID!): Step!
  }
`;