import { jest } from '@jest/globals';
import { TestPrinter } from '../index';
import { Task, Step } from '@shared/index';
import { InMemoryPrintHistoryRepository, InMemoryStepRepository } from '../../db/repositories/in-memory-repository';
import { Knex } from 'knex';

describe('Printer', () => {
  let printHistoryRepo: InMemoryPrintHistoryRepository;
  let stepRepo: InMemoryStepRepository;
  let testPrinter: TestPrinter;
  let mockDb: jest.Mocked<Knex>;

  beforeEach(() => {
    mockDb = {} as jest.Mocked<Knex>;
    printHistoryRepo = new InMemoryPrintHistoryRepository(mockDb);
    stepRepo = new InMemoryStepRepository(mockDb);
    testPrinter = new TestPrinter(printHistoryRepo, stepRepo);
  });

  it('should record print history when printing a task', async () => {
    const task: Task = { id: 1, name: 'Test Task', group_id: 1, print_count: 0, created_at: new Date(), updated_at: new Date() };
    await testPrinter.printTask(task, mockDb);
    const allHistory = await printHistoryRepo.findAll();
    expect(allHistory.length).toBe(1);
    expect(allHistory[0]).toMatchObject({
      user_id: 1,
      task_id: task.id,
      printed_at: expect.any(Date),
    });
  });

  it('should record print history when printing a step', async () => {
    const step: Step = { id: 1, name: 'Test Step', instructions: 'Test Instructions', task_id: 1, order: 1, print_count: 0, created_at: new Date(), updated_at: new Date() };
    await testPrinter.printStep(step, mockDb);
    const allHistory = await printHistoryRepo.findAll();
    expect(allHistory.length).toBe(1);
    expect(allHistory[0]).toMatchObject({
      user_id: 1,
      step_id: step.id,
      printed_at: expect.any(Date),
    });
  });
});